@@ -282,7 +282,7 @@ sub ProcessRAR($$)
         last if $size < 0;
         next unless $size;  # ignore blocks with no data
         # don't try to read very large blocks unless LargeFileSupport is enabled
-        if ($size > 0x80000000 and not $et->Options('LargeFileSupport')) {
+        if ($size >= 0x80000000 and not $et->Options('LargeFileSupport')) {
             $et->Warn('Large block encountered. Aborting.');
             last;
         }
@@ -448,7 +448,7 @@ sub ProcessZIP($$)
 
         # check for an Office Open file (DOCX, etc)
         # --> read '[Content_Types].xml' to determine the file type
-        my ($mime, @members, $epub);
+        my ($mime, @members);
         my $cType = $zip->memberNamed('[Content_Types].xml');
         if ($cType) {
             ($buff, $status) = $zip->contents($cType);
